<?php
 
 /**
  *  This provides the administration facilities for media.
  */

/**
 *  Implementation of hook_media_operations()
 */
function media_media_operations() {
  $operations = array(
    'delete' => array(
      'label' => t('Delete'),
    ),
  );
  
  return $operations;
}

/**
 * List media administration filters that can be applied.
 */
function media_filters() {
  $filters = array();
  $filters['type'] = array('title' => t('type'), 'options' => mapi_type_list());
  // The taxonomy filter
  if ($taxonomy = module_invoke('taxonomy', 'form_all', 1)) {
//    $filters['category'] = array('title' => t('category'), 'options' => $taxonomy);
  }

  return $filters;
}

function media_build_filter_query() {
  $filters = media_filters();

  // Build query
  $where = $args = array();
  $join = '';
  foreach ($_SESSION['media_overview_filter'] as $index => $filter) {
    list($key, $value) = $filter;
    switch ($key) {
      case 'type':
        $where[] = "m.type = '%s'";
				break;
    }
    $args[] = $value;
  }
  $where = count($where) ? 'WHERE '. implode(' AND ', $where) : '';

  return array('where' => $where, 'join' => $join, 'args' => $args);
}

/**
 * Return form for media administration filters.
 */
function media_filter_form() {
  $session = &$_SESSION['media_overview_filter'];
  $session = is_array($session) ? $session : array();
  $filters = media_filters();

  $i = 0;
  $form['filters'] = array('#type' => 'fieldset',
    '#title' => t('Show only items where'),
    '#theme' => 'media_filters',
  );
  foreach ($session as $filter) {
    list($type, $value) = $filter;
    if ($type == 'category') {
      // Load term name from DB rather than search and parse options array.
      $value = module_invoke('taxonomy', 'get_term', $value);
      $value = $value->name;
    }
    else {
      $value = $filters[$type]['options'][$value];
    }
    $string = ($i++ ? '<em>and</em> where <strong>%a</strong> is <strong>%b</strong>' : '<strong>%a</strong> is <strong>%b</strong>');
    $form['filters']['current'][] = array('#value' => t($string, array('%a' => $filters[$type]['title'] , '%b' => $value)));
    if ($type == 'type') {
      // Remove the type option if it is already being filtered on.
      unset($filters['type']);
    }
  }

  foreach ($filters as $key => $filter) {
    $names[$key] = $filter['title'];
    $form['filters']['status'][$key] = array('#type' => 'select', '#options' => $filter['options']);
  }

  $form['filters']['filter'] = array('#type' => 'radios', '#options' => $names, '#default_value' => 'status');
  $form['filters']['buttons']['submit'] = array('#type' => 'submit', '#value' => (count($session) ? t('Refine') : t('Filter')));
  if (count($session)) {
    $form['filters']['buttons']['undo'] = array('#type' => 'submit', '#value' => t('Undo'));
    $form['filters']['buttons']['reset'] = array('#type' => 'submit', '#value' => t('Reset'));
  }

  return $form;
}

/**
 * Theme media administration filter form.
 */
function theme_media_filter_form($form) {
  $output .= '<div id="media-admin-filter">';
  $output .= drupal_render($form['filters']);
  $output .= '</div>';
  $output .= drupal_render($form);
  return $output;
}

/**
 * Theme media administration filter selector.
 */
function theme_media_filters($form) {
  $output .= '<ul class="clear-block">';
  if (sizeof($form['current'])) {
    foreach (element_children($form['current']) as $key) {
      $output .= '<li>'. drupal_render($form['current'][$key]) .'</li>';
    }
  }

  $output .= '<li><dl class="multiselect">'. (sizeof($form['current']) ? '<dt><em>'. t('and') .'</em> '. t('where') .'</dt>' : '') .'<dd class="a">';
  foreach (element_children($form['filter']) as $key) {
    $output .= drupal_render($form['filter'][$key]);
  }
  $output .= '</dd>';

  $output .= '<dt>'. t('is') .'</dt><dd class="b">';

  foreach (element_children($form['status']) as $key) {
    $output .= drupal_render($form['status'][$key]);
  }
  $output .= '</dd>';

  $output .= '</dl>';
  $output .= '<div class="container-inline" id="media-admin-buttons">'. drupal_render($form['buttons']) .'</div>';
  $output .= '</li></ul>';

  return $output;
}

/**
 * Process result from media administration filter form.
 */
function media_filter_form_submit($form_id, $form_values) {
  $filters = media_filters();
  switch ($form_values['op']) {
    case t('Filter'):
    case t('Refine'):
      if (isset($form_values['filter'])) {
        $filter = $form_values['filter'];

        // Flatten the options array to accommodate hierarchical/nested options.
        $flat_options = form_options_flatten($filters[$filter]['options']);

        if (isset($flat_options[$form_values[$filter]])) {
          $_SESSION['media_overview_filter'][] = array($filter, $form_values[$filter]);
        }
      }
      break;
    case t('Undo'):
      array_pop($_SESSION['media_overview_filter']);
      break;
    case t('Reset'):
      $_SESSION['media_overview_filter'] = array();
      break;
  }
}

/**
 * Submit the media administration update form.
 */
function media_admin_media_submit($form_id, $form_values) {
  $operations = module_invoke_all('media_operations');
  $operation = $operations[$form_values['operation']];
  // Filter out unchecked nodes
  $nodes = array_filter($form_values['media']);
  if ($function = $operation['callback']) {
    // Add in callback arguments if present.
    if (isset($operation['callback arguments'])) {
      $args = array_merge(array($nodes), $operation['callback arguments']);
    }
    else {
      $args = array($nodes);
    }
    call_user_func_array($function, $args);

    cache_clear_all();
    drupal_set_message(t('The update has been performed.'));
  }
}

function media_admin_media_validate($form_id, $form_values) {
  $nodes = array_filter($form_values['media']);
  if (count($nodes) == 0) {
    form_set_error('', t('No items selected.'));
  }
}

/**
 * Menu callback: content administration.
 */
function media_admin_content() {
  $output = drupal_get_form('media_filter_form');

  if ($_POST['operation'] == 'delete' && $_POST['media']) {
    return drupal_get_form('media_multiple_delete_confirm');
  }
  // Call the form first, to allow for the form_values array to be populated.
  $output .= drupal_get_form('media_admin_media');

  return $output;
}

function media_admin_media() {
  $filter = media_build_filter_query();

  $result = pager_query('SELECT m.*, u.name, u.uid FROM {media} m '. $filter['join'] .' INNER JOIN {users} u ON m.uid = u.uid '. $filter['where'] .' ORDER BY m.changed DESC', 50, 0, NULL, $filter['args']);

  $form['options'] = array('#type' => 'fieldset',
    '#title' => t('Update options'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $options = array();
  foreach (module_invoke_all('media_operations') as $operation => $array) {
    $options[$operation] = $array['label'];
  }
  $form['options']['operation'] = array('#type' => 'select', '#options' => $options,  '#default_value' => 'approve');
  $form['options']['submit'] = array('#type' => 'submit', '#value' => t('Update'));

  $destination = drupal_get_destination();
  while ($obj = db_fetch_object($result)) {
    $media[$obj->mid] = '';
    $form['title'][$obj->mid] = array('#value' => l($obj->title, MEDIA_PATH .'/'. $obj->mid));
    $form['username'][$obj->mid] = array('#value' => theme('username', $obj));
    $form['type'][$obj->mid] =  array('#value' =>  $obj->type);
    $form['extension'][$obj->mid] =  array('#value' =>  $obj->ext);
    $form['operations'][$obj->mid] = array('#value' => l(t('edit'), MEDIA_PATH .'/'. $obj->mid .'/edit', array(), $destination));
  }
  $form['media'] = array('#type' => 'checkboxes', '#options' => $media);
  $form['pager'] = array('#value' => theme('pager', NULL, 50, 0));
  return $form;
}

/**
 * Theme media administration overview.
 */
function theme_media_admin_media($form) {
  // Overview table:
  $header = array(theme('table_select_header_cell'), t('Title'), t('Author'), t('Type'),  t('Extension'), t('Operations'));

  $output .= drupal_render($form['options']);
  if (isset($form['title']) && is_array($form['title'])) {
    foreach (element_children($form['title']) as $key) {
      $row = array();
      $row[] = drupal_render($form['media'][$key]);
      $row[] = drupal_render($form['title'][$key]);
      $row[] = drupal_render($form['username'][$key]);
      $row[] = drupal_render($form['type'][$key]);
      $row[] = drupal_render($form['extension'][$key]);
      $row[] = drupal_render($form['operations'][$key]);
      $rows[] = $row;
    }

  }
  else  {
    $rows[] = array(array('data' => t('No media available.'), 'colspan' => '6'));
  }

  $output .= theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);

  return $output;
}

function media_multiple_delete_confirm() {
  $edit = $_POST;

  $form['media'] = array('#prefix' => '<ul>', '#suffix' => '</ul>', '#tree' => TRUE);
  // array_filter returns only elements with TRUE values
  foreach (array_filter($edit['media']) as $mid => $value) {
    $title = db_result(db_query('SELECT title FROM {media} WHERE mid = %d', $mid));
    $form['media'][$mid] = array('#type' => 'hidden', '#value' => $mid, '#prefix' => '<li>', '#suffix' => check_plain($title) ."</li>\n");
  }
  $form['operation'] = array('#type' => 'hidden', '#value' => 'delete');

  return confirm_form($form,
                      t('Are you sure you want to delete these items?'),
                      MAPI_ADMIN_PATH .'/media', t('This action cannot be undone.'),
                      t('Delete all'), t('Cancel'));
}

function media_multiple_delete_confirm_submit($form_id, $form_values) {
  if ($form_values['confirm']) {
    foreach ($form_values['media'] as $mid => $value) {
      media_item_delete($mid);
    }
    drupal_set_message(t('The items have been deleted.'));
  }
  return MAPI_ADMIN_PATH .'/media';
}
