<?php

/**
 *  Provides a themed list of media, based on user_access and base folder.
 */
function media_list($fid = 0, $paging = 1) {
  $latest = array();

  $sql = 'SELECT mid FROM {media}';
  $order = ' ORDER BY changed DESC';
  if (user_access('view media')) {
    $results = ($paging ? pager_query($sql . $order, variable_get('media_page', 10), $paging, null) : db_query($sql . $order));
  }
  elseif (user_access('view own media')) {
    global $user;

    $sql .= ' WHERE uid=%d';
    $results = ($paging ? pager_query($sql . $order, variable_get('media_page', 10), $paging, null, $user->uid) : db_query($sql . $order, $user->uid));
  }
  else {
    $results = null;
  }

  while ($object = db_fetch_object($results)) {
    $latest[] = $object->mid;
  }

  return theme('media_list', $latest);
}

/**
 *  The add form for media items
 */
function media_add_form($type = null) {
  $edit = new stdClass;
  $edit->type = null;
  if (in_array($type, mapi_type_list($type))) {
    $edit->type = $type;
  }

  $form = _media_form($edit);

  // add the buttons
  $form['preview'] = array('#type' => 'button', '#value' => t('Preview'));
  $form['submit'] = array('#type' => 'submit', '#value' => t('Submit'));

  return $form;
}

function media_add_form_validate($form_id, $form_values) {
  _media_form_process($form_values);
}

function media_add_form_submit($form_id, $form_values) {
  $media = _media_form_post($form_values);
  if ($media->mid) {
    return MEDIA_PATH .'/'. $media->mid .'/view';
  }
}

/**
 *  The edit form for media items
 */
function media_edit_form($mid) {
  $media = media_item_load($mid);
  $media->filename = media_item_filename($media);
  $media->metadata = media_metadata_load($mid);

  $form = _media_form($media);

  // add the buttons
  $form['preview'] = array('#type' => 'button', '#value' => t('Preview'));
  $form['submit'] = array('#type' => 'submit', '#value' => t('Submit'));
  $form['delete'] = array('#type' => 'button', '#value' => t('Delete'));

  return $form;
}

function media_edit_form_validate($form_id, $form_values) {
  if ($form_values['op'] == t('Delete')) {
    drupal_goto(MEDIA_PATH .'/'. $form_values['mid'] .'/delete');
    exit();
  }

  _media_form_process($form_values);
}

function media_edit_form_submit($form_id, $form_values) {
  $media = _media_form_post($form_values);
  return MEDIA_PATH .'/'. $media->mid .'/view';
}

/**
 *  The delete form for media items
 */
function media_delete_form($mid) {
  // load the media item
  $media = media_item_load($mid);

  // remember for confirmation of the media
  $form['mid'] = array('#type' => 'value', '#value' => $mid);

  // provide user interaction
  $notice = t('This action cannot be undone.');

  $form = confirm_form($form,
    t('Are you sure you want to delete %title?', array('%title' => $media->title)),
    isset($_GET['destination']) ? $_GET['destination'] : MEDIA_PATH ."/{$mid}/view",
    $notice,
    t('Delete'), t('Cancel'));

  return $form;
}

/**
 *  Submission for media delete
 */
function media_delete_form_submit($form_id, $form_values) {
  if ($form_values['confirm'] == 1) {
    $mid = $form_values['mid'];
    media_item_delete($mid);
  }

  return (isset($_GET['destination']) ? $_GET['destination'] : '<front>');
}

/**
 *  The move form for media items
 */
function media_move_form($mid) {
  drupal_add_css(drupal_get_path('module', 'media') .'/media.css');

  $media = media_item_load($mid);

  $folders = _media_folder_cache();

  // add everything else
  $current = 0;
  $lines = $folders['line'];
  foreach ($lines as $line => $fid) {
    $depth = (count(explode('/', $line)) - FOLDER_PATH_ARG - 1);

    // add the appropriate boundaries for accordian viewing
    $prefix = '';
    while ($depth > $current++) {
      $prefix .= '<div class="folder-level">';
    }
    while ($current > $depth + 1) {
      $prefix .= '</div>';
      $current--;
    }

    //
    $form['radios'][$line] = array(
      '#prefix' => $prefix . '<div class="folder-level-'. $depth .'">',
      '#type' => 'radio',
      '#title' => $folders[$fid]['name'],
      '#return_value' => $fid,
      '#default_value' => ($fid == $media->fid ? $fid : false),
      '#parents' => array('folder'),
      '#suffix' => '</div>'
    );
    $current = $depth;
  }
  $form['radios']['#suffix'] = '';
  while ($current > 0) {
    $form['radios']['#suffix'] .= '</div>';
    $current--;
  }

  $form['submit'] = array('#type' => 'submit', '#value' => t('Submit'));
  $form['mid'] = array('#type' => 'value', '#value' => $mid);

  return $form;
}

function media_move_form_submit($form_id, $form_values) {
  if ($form_values['op'] == t('Submit')) {
    media_item_move($form_values['mid'], $form_values['folder']);
  }
}

/**
 *  The view form for media items
 */
function media_view_form($mid) {
  $media = media_item_load($mid);

  // allow various other modules to do things to view
  $display = media_invoke_item('view', $media);
  if (!$display) {
    $display = array('#value' => mapi_display(media_item_filename($media), array('context' => 'full')));
  }

  drupal_set_title(t($media->title));

  // allows for overriding the view form easily
  $form['#media'] = $media;

  // display the result
  $form['display'] = array(
    '#prefix' => '<div class="media-content">',
    '#suffix' => '</div>',
  );

  $form['display'] = array_merge($form['display'], $display);

  $display = media_invoke_metadata('view', media_metadata_load($mid));
  if ($display) {
    $form['metadata'] = array(
      '#prefix' => '<div class="media-content">',
      '#suffix' => '</div>',
    );

    $form['metadata'] = array_merge($form['metadata'], $display);
  }

  return $form;
}

/**
 *  The media derivatives form
 */
function media_derivatives_form($mid, $did) {
  $media = media_item_load($mid);

  // figure out if this is a derivative that this media item
  if ($did && ($derivative = mapi_derivative_load($did)) && (in_array($media->ext, explode('|', $derivative->exts)))) {
    // get the details of the generated derivative
    $details = mapi_generated_details(media_item_filename($media), $derivative->name);

    // basic necessities for other modules to create changes
    $form['mid'] = array('#type' => 'value', '#value' => $mid);
    $form['did'] = array('#type' => 'value', '#value' => $did);

    if ($details) {
      $form['preview'] = array('#value' => '<div class="media-derivative-preview">'. mapi_display($details['filename']) .'</div>');
    }
    else {
      $form['preview'] = array('#value' => t('There is no generated derivative for this media.') .'<p />');
    }

    // delete derivative
    $form['delete'] = array('#type' => 'submit', '#value' => t('Delete Derivative'));

    // force generation of derivative
    $form['force'] = array('#type' => 'submit', '#value' => t('Force Generation'));
  }
  // get list of derivatives for current media derivative
  else {
    $derivatives = mapi_derivatives_by_ext($media->ext);

    $list = array();
    foreach ($derivatives as $did => $name) {
      $list[] = l($name, MEDIA_PATH .'/'. $mid .'/derivatives/'. $did);
    }

    if (count($list)) {
      $form['details'] = array('#value' => theme('item_list', $list));
    }
    else {
      $form['details'] = array('#value' => t('There are no derivatives that can be generated for this media.'));
    }
  }

  return $form;
}

function media_derivatives_form_submit($form_id, $form_values) {
  switch ($form_values['op']) {
    // deletes the derivative
    case t('Delete Derivative'):
      $media = media_item_load($form_values['mid']);
      $derivative = mapi_derivative_name($form_values['did']);
      mapi_generated_delete(media_item_filename($media), $derivative);
      break;

    // forces a regeneration of the file if necessary
    case t('Force Generation'):
      $media = media_item_load($form_values['mid']);
      $derivative = mapi_derivative_name($form_values['did']);

      // check for already generated file, replace it.
      $filename = null;
      if ($generated = mapi_generated_details(media_item_filename($media), $derivative)) {
        $filename = $generated['filename'];
      }

      // generates the derivative
      mapi_derive(media_item_filename($media), $filename, $derivative, array('exists' => FILE_EXISTS_REPLACE));
      break;
  }
}

/**
 *   Provides a list of folders, based on user_access and base folder.
 */
function media_folder_list($fid = 0) {
  global $user;

  $list = array();

  $folder = media_folder_load($fid);
  if ($folder->fid != $folder->parent) {
    $list[] = l('..', FOLDER_PATH .'/'. rtrim($folder->path, '/'));
  }

  $folders = _media_folder_cache();
  $children = media_folder_subfolders($fid, $user->uid);
  foreach ($children as $id => $child) {
    $list[] = l($child, FOLDER_PATH .'/'. $folders[$id]);
  }

  return theme('item_list', $list);
}

/**
 *  The add form for media items
 */
function media_folder_add_form($fid = 0) {
  $edit = new stdClass;
  $edit->parent = $fid;
  return _media_folder_form($edit);
}

function media_folder_add_form_validate($form_id, $form_values) {
  _media_folder_process($form_values);
}

function media_folder_add_form_submit($form_id, $form_values) {
  $folder = _media_folder_post($form_values);
  if ($folder->fid) {
    drupal_goto(FOLDER_PATH .'/'. rtrim($folder->path, '/'));
    exit();
  }
}

/**
 *  Form for editing folders
 */
function media_folder_edit_form($fid) {
  $folder = media_folder_load($fid);
  return _media_folder_form($folder);
}

function media_folder_edit_form_validate($form_id, $form_values) {
  _media_folder_process($form_values);
}

function media_folder_edit_form_submit($form_id, $form_values) {
  $folder = _media_folder_post($form_values);
  if ($folder->fid) {
    drupal_goto(FOLDER_PATH .'/'. rtrim($folder->path, '/'));
    exit();
  }
}

/**
 *  Deletion of folders form
 */
function media_folder_delete_form($fid) {
  // load the media folder
  $folder = media_folder_load($fid);

  // remember for confirmation of the media
  $form['fid'] = array('#type' => 'value', '#value' => $fid);
  if (user_access('delete media')) {
    $form['delete'] = array('#type' => 'checkbox', '#title' => t('Delete contained media'));
  }

  // provide user interaction
  $notice = t('This action cannot be undone.');
  $notice .= ' '. t('It will delete all sub-folders.');

  $form = confirm_form($form,
    t('Are you sure you want to delete %title?', array('%title' => $folder->name)),
    isset($_GET['destination']) ? $_GET['destination'] : FOLDER_PATH .'/'. rtrim($folder->path, '/'),
    $notice,
    t('Delete'), t('Cancel'));

  return $form;
}

/**
 *  Submission for folder delete
 */
function media_folder_delete_form_submit($form_id, $form_values) {
  if ($form_values['confirm'] == 1) {
    $fid = $form_values['fid'];
    $delete = (isset($form_values['delete']) ? $form_values : false);
    media_folder_delete($fid, $delete);
  }

  return (isset($_GET['destination']) ? $_GET['destination'] : '<front>');
}

/**
 *  Folders have to move
 */
function media_folder_move_form($fid) {
  drupal_add_css(drupal_get_path('module', 'media') .'/media.css');

  $folder = media_folder_load($fid);

  $folders = _media_folder_cache();
  // add base folder
  $form['radios'][''] = array(
    '#prefix' => '<div class="folder-level-0">',
    '#type' => 'radio',
    '#title' => t('none'),
    '#return_value' => 0,
    '#default_value' => ($folder->parent == 0 ? 0 : false),
    '#parents' => array('folder'),
    '#suffix' => '</div>'
  );

  // add everything else
  $base = FOLDER_PATH .'/'. $folders[$fid]['line'];
  $current = 0;
  $lines = $folders['line'];
  ksort($lines);
  foreach ($lines as $line => $parent) {
    // ignore all of those which are either this folder or sub-folders
    if (strpos($line, $base) === 0) {
      continue;
    }

    $depth = (count(explode('/', $line)) - 2);

    // add the appropriate boundaries for accordian viewing
    $prefix = '';
    while ($depth > $current++) {
      $prefix .= '<div class="folder-level">';
    }
    while ($current > $depth + 1) {
      $prefix .= '</div>';
      $current--;
    }

    //
    $form['radios'][$line] = array(
      '#prefix' => $prefix . '<div class="folder-level-'. $depth .'">',
      '#type' => 'radio',
      '#title' => $folders[$parent]['name'],
      '#return_value' => $parent,
      '#default_value' => ($parent == $folder->parent ? $parent : false),
      '#parents' => array('folder'),
      '#suffix' => '</div>'
    );
    $current = $depth;
  }
  $form['radios']['#suffix'] = '';
  while ($current > 0) {
    $form['radios']['#suffix'] .= '</div>';
    $current--;
  }

  $form['submit'] = array('#type' => 'submit', '#value' => t('Submit'));
  $form['fid'] = array('#type' => 'value', '#value' => $fid);

  return $form;
}

function media_folder_move_form_submit($form_id, $form_values) {
  if ($form_values['op'] == t('Submit')) {
    media_folder_move($form_values['fid'], $form_values['folder']);
  }
}

/**
 *  Viewing the folder
 */
function media_folder_view_form($fid) {
  $folder = media_folder_load($fid);

  // allow various other modules to do things to view
  $display = media_invoke_folder('view', $folder);
  if (!$display) {
    // create a list of the media within this folder
    // use the paging mechanism
    $display = array();

    // get a list of children for the folder using paging
    $children = media_folder_children($fid, 1);

    // add pager to the top
    if ($pager = theme('pager', NULL, variable_get('media_folder_page', 10), 1)) {
      $display[] = array('#value' => '<div class="media-pager">'. $pager .'</div>');
    }

    // generate the display, possibly with extra things in folder view
    $rows = array();
    foreach ($children as $mid => $title) {
      $row = array();
      $details = media_invoke_item('folder-view', media_item_load($mid));
      if (is_array($details)) {
        foreach ($details as $detail) {
          $row[] = $detail;
        }
      }
      $row[] = l($title, MEDIA_PATH ."/{$mid}");
      $rows[] = $row;
    }

    $display[] = array('#value' => theme('table', array(), $rows));

    // add pager to the bottom
    if ($pager) {
      $display[] = array('#value' => '<div class="media-pager">'. $pager .'</div>');
    }
  }

  drupal_set_title(t($folder->name));

  // display the result
  $form['display'] = array(
    '#prefix' => '<div class="folder-content">',
    '#suffix' => '</div>',
  );

  $form['display'] = array_merge($form['display'], $display);

  return $form;
}

function _media_folder_form($edit = null) {
  $form['#folder'] = $edit;

  // basic values which need to be passed over to this
  $form['fid'] = array('#type' => 'value', '#value' => $edit->fid);
  $form['uid'] = array('#type' => 'value', '#value' => $edit->uid);
  $form['parent'] = array('#type' => 'value', '#value' => $edit->parent);

  // name
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#required' => true,
    '#maxlength' => 255,
    '#default_value' => $edit->name
  );

  $form['submit'] = array('#type' => 'submit', '#value' => t('Submit'));
  if ($edit->fid) {
    $form['delete'] = array('#type' => 'button', '#value' => t('Delete'));
  }

  return $form;
}

function _media_folder_process($form_values) {
  // checks that name exists
  if ($form_values['name'] && preg_match('/[^\w\d_\- ]+/', $form_values['name'])) {
    form_set_error('name', t('Name can only contain alphanumeric, space, hyphen and underscore characters.'));
  }
  elseif (media_folder_exists($form_values['names'], (int)$form_values['parent'], (int)$form_values['fid'])) {
    form_set_error('name', t('Folder of that name already exists.'));
  }
  elseif ($form_values['op'] == t('Delete')) {
    $folders = _media_folder_cache();
    if (array_key_exists($form_values['fid'], $folders)) {
      $folder = $folders[$form_values['fid']];
      if (is_array($folder)) {
        drupal_goto(FOLDER_PATH .'/'. $folder['line'] . 'delete');
        exit();
      }
    }
  }
}

function _media_folder_post($form_values) {
  // simply rename the folder
  if ($fid = $form_values['fid']) {
    $folder = media_folder_load($fid);
    media_folder_rename($fid, $form_values['name']);
  }
  // otherwise create a new folder
  else {
    $folder = media_folder_create((int)$form_values['parent'], $form_values['name']);
  }
  return $folder;
}

/**
 *  Autocomplete functions
 */
function media_autocomplete($string) {
  if ($string != '') {
    $result = db_query_range('SELECT mid, title FROM {media} WHERE LOWER(title) LIKE LOWER("%%%s%%")', strtolower($string), 0, 10);

    $matches = array();
    while ($media = db_fetch_object($result)) {
      $t = $media->title;
      // Commas and quotes in terms are special cases, so encode 'em.
      if (preg_match('/,/', $media->title) || preg_match('/"/', $media->title)) {
        $t = '"'. preg_replace('/"/', '""', $media->title) .'"';
      }
      $matches[$media->mid] = '<span class="autocomplete_title">'. check_plain($media->title) .'</span> <span class="autocomplete_mid">[mid:'. $media->mid .']</span>';
    }
    print drupal_to_js($matches);
    exit();
  }
}

function media_folder_autocomplete($string) {
  if ($string != '') {
    $result = db_query_range('SELECT fid, name, path FROM {media_folders} WHERE LOWER(name) LIKE LOWER("%%%s%%")', strtolower($string), 0, 10);

    $matches = array();
    while ($folder = db_fetch_object($result)) {
      $t = $folder->name;
      // Commas and quotes in terms are special cases, so encode 'em.
      if (preg_match('/,/', $folder->name) || preg_match('/"/', $folder->name)) {
        $t = '"'. preg_replace('/"/', '""', $folder->name) .'"';
      }
      $matches[$folder->fid] = '<span class="autocomplete_title">'. check_plain($folder->name) .'</span> <span class="autocomplete_path">['. FOLDER_PATH .'/'. $folder->path .']</span>';
    }
    print drupal_to_js($matches);
    exit();
  }
}

/**
 *  Theming forms
 */
function theme_media_list($items) {
  $output = '';
  $rows = array();
  if (is_array($items)) {
    foreach ($items as $item) {
      if (is_numeric($item)) {
        $media = media_item_load($item);
        $display = media_invoke_item('view', $media);
        if (!$display) {
          $display = array('#value' => mapi_display(media_item_filename($media), array('context' => 'teaser')));
        }
        $row = array(
          '<div class="media-display">'. drupal_render($display) .'</div>',
          '<div class="media-title">'. l($media->title, MEDIA_PATH .'/'. $media->mid) .'</div>',
        );

        $rows[] = array('data' => $row, 'class' => 'media-item');
      }
      elseif (is_string($item) && media_file_exists($item)) {
        $row = array(
          '<div class="media-display">'. mapi_display($item, array('context' => 'teaser')) .'</div>',
          ''
        );

        $rows[] = array('data' => $row, 'class' => 'media-item');
      }
    }
  }

  return theme('table', array(), $rows, array('style' => 'width: 100%'));
}