<?php

/**
 *  Implementation of hook_media_attach()
 */
function mapi_audio_media_attach($op, &$item, $data) {
  if ($op == 'embed') {
    return array(
      'audio' => array('_mapi_audio_media_attach_audio', $data, $item),
      'playlist' => array('_mapi_audio_media_attach_playlist', $data, $item),
    );
  }
}

function _mapi_audio_media_attach_audio($list, $node, $context) {
  $attached = media_attach_node_defaults($node, array('type' => 'audio'));

  // extract settings from list
  $items = (isset($list[0]) ? $list[0] : array());
  $settings = (isset($list[1]) ? $list[1] : array());

  // get the first of the list, if there is nothing already selected
  if (($index = array_shift($items)) && array_key_exists($index, $attached)) {
    $index = $attached[$index];
  }
  else {
    $index = array_shift($attached);
  }

  if (!empty($index)) {
    $filename = $index['filename'];

    // check if we can use the layouting module
    $layout = (isset($settings['layout']) ? $settings['layout'] : '');
    if (module_exists('mapi_layout') && $layout && in_array($layout, mapi_layout_list())) {
      // load the metadata and add necessary outlets
      $meta = media_metadata_load($index['mid']);
      $meta['caption']  = $index['caption'];
      $meta['filename'] = $index['filename'];
      $meta['content']  = mapi_display($index['filename']);
      $meta['context']  = $context;
      $meta['settings'] = $settings;

      // run it through the layout generator
      $display = mapi_layout_generate($layout, $meta);
    }
    else {
      $display = mapi_display($filename, array('context' => $context));
    }

    return $display;
  }
}

function _mapi_audio_media_attach_playlist($list, $node, $context) {
$attached = media_attach_node_playlist($node, array('type' => 'audio'));

  // extract settings from list
  $items = (isset($list[0]) ? $list[0] : array());
  $settings = (isset($list[1]) ? $list[1] : array());

  // remove all items that are NOT in the list, otherwise all are displayed
  if (!empty($items)) {
    foreach (array_keys($attached) as $key) {
      if (!in_array($key, $items)) {
        unset($attached[$key]);
      }
    }
  }

  // no playlist of movies, return nothing
  if (!count($attached)) {
    return '';
  }

  $first = $attached[0]['filename'];
  if (!isset($settings['height']) || !isset($settings['width'])) {
    $dim = mapi_file_dimensions($first);
    // we want to keep the general aspect ratio here
    if (isset($dim['height']) && $dim['height'] && isset($dim['width']) && $dim['width']) {
      if (!isset($settings['height']) && !isset($settings['width'])) {
        $settings['height'] = $dim['height'];
        $settings['width'] = $dim['width'];
      }
      elseif (!isset($settings['height'])) {
        $settings['height'] = ($dim['height']/$dim['width']) * $settings['width'];
      }
      else {
        $settings['width'] = ($dim['width']/$dim['height']) * $settings['height'];
      }
    }
  }

  // get the presenter
  $presenter = (isset($settings['presenter']) ? $settings['presenter'] : null);
  if (!in_array($presenter, mapi_presenter_list())) {
    $presenter = mapi_presenter_default(mapi_file_ext($first));
  }
  // we need a valid presenter
  if (!in_array($presenter, mapi_presenter_list())) {
    return '';
  }

  // generate the appropriate playlist filename
  $filename = url('node/'. $node->nid .'/media/playlist/'. $presenter .'/audio/'. implode('-', array_keys($attached)));

  $display = mapi_display_playlist($filename, $presenter, $settings);

   // check if we can use the layouting module
  $layout = (isset($settings['layout']) ? $settings['layout'] : '');
  if (module_exists('mapi_layout') && $layout && in_array($layout, mapi_layout_list())) {
    // provide standard details
    $meta['filename'] = $image['filename'];
    $meta['content'] = $display;
    $meta['context'] = $context;
    $meta['settings'] = $settings;

    // run it through the layout generator
    $display = mapi_layout_generate($layout, $meta);
  }

  return $display;
}
